@section('title', 'VisiMisi')

@extends('Frontend.Layouts.app2')
@section('content')
<br><br>
    <x-header-section-page title="Visi Misi {{ $settings['nama kampus'] ?? 'Nama kampus tidak tersedia' }}" breadcrumb-home="Home" breadcrumb-current="Visi Misi" />

    <!-- Content Section -->
    <div class=" py-10 ">
        <div data-aos="fade-up" class="container mx-auto px-4 lg:px-16 space-y-12">
            <!-- Visi -->
            <div data-aos="fade-up" class="bg-white shadow-xl shadow rounded-lg p-6">
                <h2 data-aos="fade-up" class="text-xl font-poppins font-semibold text-primary mb-3">Visi </h2>
                <p data-aos="fade-up" class="text-gray-700 font-dmsans">
                   Menjadi Perguruan Tinggi unggul yang menghasilkan insan profesional, mandiri, berakhlak mulia, berbudaya berbasis kearifan lokal pada Tahun 2028
                </p>

            </div>

            <!-- Misi -->
            <div data-aos="fade-up" class="bg-white shadow-xl shadow rounded-lg p-6">
                <h2 data-aos="fade-up" class="text-xl font-poppins font-semibold text-primary mb-3">Misi</h2>
                <ul data-aos="fade-up" class="list-disc pl-5 space-y-3 text-gray-700 font-dmsans">
                   <li>Menyelenggarakan pendidikan berkualitas yang berbasis riset dan perkembangan sains, teknologi, sosial, dan berbasis kearifan lokal.</li>        
                   <li>Menyelenggarakan penelitian untuk mengembangkan ilmu dan teknologi yang mengedepankan nilai-nilai budaya pada masyarakat.</li>        
                   <li>Menyelenggarakan pengabdian kepada masyarakat untuk mengembangkan ilmu dan teknologi dengan pemberdayaan masyarakat berbasis kearifan lokal.</li>        
                   <li>Mampu melaksanakan Tridharma Perguruan Tinggi secara seimbang.</li>     
                </ul>
            </div>

            <!-- Call to Action -->
            <div data-aos="fade-up" class="text-center shadow-xl">
                <div data-aos="fade-up" class="bg-accent py-8 rounded-lg" style="color:#fff">
                    <h2 data-aos="fade-up" class="text-xl font-poppins font-bold  mb-3">Bergabunglah dengan {{ $settings['nama kampus'] ?? 'Nama kampus tidak tersedia' }}
                    </h2>
                    <p data-aos="fade-up" class=" font-dmsans mb-5">Wujudkan impianmu dan raih masa depan cerah bersama {{ $settings['nama kampus'] ?? 'Nama kampus tidak tersedia' }}</p>
                    <a href="/pendaftaran" data-aos="fade-up"
                        class="bg-yellow-400 hover:bg-primary font-medium py-2 px-3 rounded-md text-sm transition-all text-center text-white">
                        Daftar Sekarang
                    </a>
                </div>
            </div>
        </div>
    </div>
@endsection
