* :doc:`/cookbook/default_expectations`
* :doc:`/cookbook/detecting_mock_objects`
* :doc:`/cookbook/not_calling_the_constructor`
* :doc:`/cookbook/mocking_hard_dependencies`
* :doc:`/cookbook/class_constants`
* :doc:`/cookbook/big_parent_class`
* :doc:`/cookbook/mockery_on`
