<nav class="sidebar">
    <div class="sidebar-header">
        <a href="/" class="d-flex justify-content-center align-items-center">
            <img src="https://kuliahkaryawan.net/assets/images/k2-final-logo.png" width="30" alt="Kuliah Karyawan">
            <h5 class="mx-2">x</h5>
            <img width="30" height="30"
                    src="<?php echo e(Auth::user()->gambar ? asset('storage/userImage/' . Auth::user()->gambar) : asset('assets/user/profil.jpg')); ?>"
                    alt="profile">
        </a>

        <div class="sidebar-toggler not-active">
            <span></span>
            <span></span>
            <span></span>
        </div>
    </div>
    <div class="sidebar-body">
        <ul class="nav">
            <li class="nav-item nav-category">Menu</li>
            
            
            <?php if(Auth::user()->canAccessMenu('dashboard')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('dashboard') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('dashboard')); ?>">
                    <i class="fa fa-id-card-o" aria-hidden="true"></i>
                    <span class="link-title">Dashboard</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('daftar')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('daftar', 'daftar.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('daftar')); ?>">
                    <i class="fa fa-book" aria-hidden="true"></i>
                    <span class="link-title">Data Pendaftaran</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('tracer')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('tracer.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('tracer.index')); ?>">
                    <i class="fa fa-line-chart" aria-hidden="true"></i>
                    <span class="link-title">Tracer Study</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('menu')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('menu', 'menu.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('menu')); ?>">
                    <i class="fa fa-minus-square" aria-hidden="true"></i>
                    <span class="link-title">Pengaturan Menu</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('pages')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('pages', 'pages.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('pages')); ?>">
                    <i class="fa fa-file-text" aria-hidden="true"></i>
                    <span class="link-title">Pengaturan Halaman</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('gallery')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('gallery.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('gallery.index')); ?>">
                    <i class="fa fa-picture-o" aria-hidden="true"></i>
                    <span class="link-title">Pengaturan Galeri Foto</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('settings')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('settings', 'settings.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('settings')); ?>">
                    <i class="fa fa-server" aria-hidden="true"></i>
                    <span class="link-title">Pengaturan Umum</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('user')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('user.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('user.index')); ?>">
                    <i class="fa fa-user-o" aria-hidden="true"></i>
                    <span class="link-title">Pengaturan Admin</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('berita')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('berita', 'berita.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('berita')); ?>">
                    <i class="fa fa-newspaper-o" aria-hidden="true"></i>
                    <span class="link-title">Berita Terkini</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('biaya')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('biaya.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('biaya.index')); ?>">
                    <i class="fa fa-calculator" aria-hidden="true"></i>
                    <span class="link-title">Pengaturan Biaya Kuliah</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('brosur')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('brosur.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('brosur.index')); ?>">
                    <i class="fa fa-file" aria-hidden="true"></i>
                    <span class="link-title">Pengaturan Brosur</span>
                </a>
            </li>
            <?php endif; ?>

            
            <?php if(Auth::user()->canAccessMenu('kurikulum')): ?>
            <li class="nav-item <?php echo e(request()->routeIs('kurikulum.*') ? 'active' : ''); ?>">
                <a class="nav-link" href="<?php echo e(route('kurikulum.index')); ?>">
                    <i class="fa fa-graduation-cap" aria-hidden="true"></i>
                    <span class="link-title">Pengaturan Kurikulum</span>
                </a>
            </li>
            <?php endif; ?>
        </ul>
    </div>
</nav><?php /**PATH /home/edunesiaserver/public_html/K2/WEB_KAMPUS/WEB_TEMPLATE/Ijo-IjoTua/resources/views/dashboard/sidebar.blade.php ENDPATH**/ ?>