<!DOCTYPE html>
<html lang="id">

<head>
    <meta charset="utf-8">
    <meta name="description" content="Institut Az Zuhra adalah perguruan tinggi berbasis nilai-nilai Islami yang berlokasi di Riau. Az Zuhra menawarkan berbagai program studi unggulan jenjang S1, seperti S1 Manajemen, Akuntansi, Pendidikan Bahasa Inggris, dan Pendidikan Matematika.Serta jenjang D3, seperti D3 Ilmu Komputer, dan Manajemen Informatika. Dengan biaya kuliah yang terjangkau, Az Zuhra menjadi pilihan tepat bagi calon mahasiswa yang mencari pendidikan berkualitas, lingkungan akademik yang mendukung, serta pengembangan karakter islami. Dapatkan pengalaman belajar yang modern, relevan dengan kebutuhan dunia kerja, dan bernilai keislaman di Az ZUhra.">
    <meta name="keywords" content="edunitas, Kelas karyawan, Kuliah karyawan, INSTITA, kampus INSTITA , biaya kuliah kampus Institut Elkatarie , biaya kuliah, kelas sabtu, akreditasi Institut Elkatarie, kelas karyawan terjangkau Lombok , S1 Manajemen, Akuntansi, Pendidikan Bahasa Inggris, dan Pendidikan Matematika.Serta jenjang D3, seperti D3 Ilmu Komputer, dan Manajemen Informatika, kampus murah Di Riau , kampus Di Riau , Institut Elkatarie, biaya kuliah, kelas sabtu, akreditasi Az Zuhra ,
     Kampus Kelas Karyawan Di Lombok , Kuliah Sambil Kerja Di Lombok , Universitas Terbaik di Lombok  untuk Karyawan, Biaya Kuliah Terjangkau di Lombok , Universitas Kelas Sabtu Minggu Di Lombok , Akreditasi Universitas Di Riau ,Perguruan Tinggi Swasta Murah di Riau,  Kampus Swasta Terbaik Riau , Universitas Murah  Di Riau  terakreditasi Baik,
     Kampus Terjangkau di Lombok  untuk Karyawan, , Kuliah dengan Biaya Murah di Riau  , Universitas dengan Kelas Online Riau , Kampus Swasta Kelas Sabtu Minggu di Riau , Biaya Kuliah  Sarjana Murah di Riau , Biaya Kuliah S1 Murah di Riau  , Kampus dengan Program Studi Lengkap di Lombok " />
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <!--ADS GSC-->

    
    <title><?php echo $__env->yieldContent('title'); ?> &mdash; Institut Az Zuhra (Kelas Karyawan)</title>
    <meta property="og:title" content="Institut Az Zuhra (Kelas Karyawan)">
    <meta property="og:type" content="website">
    <meta property="og:url" content="Institut Az Zuhra (Kelas Karyawan)">
    <meta property="og:image" content="https://institutazzuhra.info/images/background/hero1.png">
    <meta property="og:image:width" content="1200" />
    <meta property="og:image:height" content="630" />
    <meta property="og:description" content="Institut Az Zuhra di Kota Riau menawarkan program S1 Manajemen, Akuntansi, Pendidikan Bahasa Inggris, dan Pendidikan Matematika.Serta jenjang D3, seperti D3 Ilmu Komputer, dan Manajemen Informatika dengan biaya kuliah terjangkau. Pilihan tepat untuk pendidikan berkualitas berbasis nilai-nilai Islami.">
    <meta name="twitter:card" content="summary_large_image">
    <meta name="twitter:creator" content="@_kuliahkaryawan">
    <meta name="twitter:site" content="@_kuliahkaryawan">
    <meta name="twitter:title" content="Institut Az ZUhra (Kelas Karyawan)">
    <meta name="twitter:description" content="Institut Az Zuhra di Kota Riau menyediakan program Kelas Karyawan dengan biaya kuliah terjangkau untuk jenjang S1 Manajemen, Akuntansi, Pendidikan Bahasa Inggris, dan Pendidikan Matematika.Serta jenjang D3, seperti D3 Ilmu Komputer, dan Manajemen Informatika. Dirancang khusus bagi pekerja dan profesional yang ingin melanjutkan pendidikan tanpa mengganggu aktivitas kerja, Az Zuhra menawarkan jadwal fleksibel, suasana akademik islami, serta kurikulum yang relevan dengan kebutuhan dunia kerja. Raih gelar sarjana sambil tetap produktif bersama Az Zuhra Kelas Karyawan!" />
    <meta name="twitter:domain" content="   ">
    <meta name="twitter:image:src" content="https://institutazzuhra.info/images/background/hero1.png">
    <meta name="author" content="Kuliah Karyawan Team" />
    <meta http-equiv="cache-control" content="public" />

    <link rel="apple-touch-icon" sizes="57x57" href="/images/favicons/apple-icon-57x57.png">
    <link rel="apple-touch-icon" sizes="60x60" href="/images/favicons/apple-icon-60x60.png">
    <link rel="apple-touch-icon" sizes="72x72" href="/images/favicons/apple-icon-72x72.png">
    <link rel="apple-touch-icon" sizes="76x76" href="/images/favicons/apple-icon-76x76.png">
    <link rel="apple-touch-icon" sizes="114x114" href="/images/favicons/apple-icon-114x114.png">
    <link rel="apple-touch-icon" sizes="120x120" href="/images/favicons/apple-icon-120x120.png">
    <link rel="apple-touch-icon" sizes="144x144" href="/images/favicons/apple-icon-144x144.png">
    <link rel="apple-touch-icon" sizes="152x152" href="/images/favicons/apple-icon-152x152.png">
    <link rel="apple-touch-icon" sizes="180x180" href="/images/favicons/apple-icon-180x180.png">
    <link rel="icon" type="image/png" sizes="192x192"  href="/images/favicons/android-icon-192x192.png">
    <link rel="icon" type="image/png" sizes="32x32" href="/images/favicons/favicon-32x32.png">
    <link rel="icon" type="image/png" sizes="96x96" href="/images/favicons/favicon-96x96.png">
    <link rel="icon" type="image/png" sizes="16x16" href="/images/favicons/favicon-16x16.png">
    <link rel="manifest" href="/images/favicons/manifest.json">
    <meta name="msapplication-TileColor" content="#ffffff">
    <meta name="msapplication-TileImage" content="/images/favicons/ms-icon-144x144.png">
    <meta name="theme-color" content="#ffffff">
    
    <link href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0/css/all.min.css" rel="stylesheet">
    <link href="https://cdn.jsdelivr.net/npm/flowbite@2.5.2/dist/flowbite.min.css" rel="stylesheet" />
    <link href="https://fonts.googleapis.com/css2?family=Poppins:wght@400;600&family=DM+Sans:wght@400;700&display=swap"
        rel="stylesheet">
    <link href="./css/app.css" rel="stylesheet">
    <link href="<?php echo e(asset('css/app.css')); ?>" rel="stylesheet">
    
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/swiper@11/swiper-bundle.min.css" />
    
    <link href="https://cdn.jsdelivr.net/npm/aos@2.3.4/dist/aos.css" rel="stylesheet">
    
    <?php echo $__env->yieldPushContent('prpend-style'); ?>
    <?php echo $__env->yieldPushContent('addon-style'); ?>

</head>

<body>

    
    <?php echo $__env->make('Frontend.Template.navbar', \Illuminate\Support\Arr::except(get_defined_vars(), ['__data', '__path']))->render(); ?>

    
    
    <div class="min-h-screen bg-cover bg-no-repeat bg-right md:bg-center-top"
        style="background-image: url('<?php echo e(asset('./images/background/bg-1.png')); ?>');">
        
        <div class="w-full"> 
            <?php echo $__env->yieldContent('content'); ?>
        </div>
    </div>

    <div class="fixed flex-row items-center bottom-4 right-4 z-50 md:bottom-10 md:right-10 flex ">
        <!-- Animated WhatsApp Button -->
        <div style="color:#fff;"
            class="slide-text whatsapp-button overflow-hidden transition-all duration-500 ease-in-out bg-[#40C351] h-[40px] py-2 px-4 font-poppins text-white rounded-full mb-2">
            <h1 class="whatsapp-button-text ">ingin bertanya ? yuk di click !</h1>
        </div>

        <!-- WhatsApp Icon -->
        <a href="https://wa.me/6289504586704?text=Halo%20Admin%2C%20saya%20mau%20bertanya%20%0ASumber%20%3A%20<?php echo e(url()->current()); ?>"
            target="_blank" rel="noopener noreferrer">
            <img src="<?php echo e(asset('images/logo/wa.jpg')); ?>" alt="WhatsApp" class="w-12 h-12 md:w-[70px] md:h-[70px]">
        </a>
    </div>

    
    <?php echo $__env->make('Frontend.Template.footer', \Illuminate\Support\Arr::except(get_defined_vars(), ['__data', '__path']))->render(); ?>



    
    <?php echo $__env->yieldPushContent('prpend-script'); ?>
    <?php echo $__env->yieldPushContent('addon-script'); ?>



    
    <script src="https://www.google.com/recaptcha/api.js"></script>
    
    <script src="https://cdn.jsdelivr.net/npm/aos@2.3.4/dist/aos.js"></script>
    
    <script src="https://cdn.jsdelivr.net/npm/flowbite@2.5.2/dist/flowbite.min.js"></script>
    <script src="https://cdn.jsdelivr.net/npm/swiper@11/swiper-bundle.min.js"></script>
    <script src="./js/app.js"></script>
    <script src="<?php echo e(asset('./js/app.js')); ?>"></script>
    <script>
document.addEventListener("DOMContentLoaded", () => {
    const modal = document.getElementById("recaptcha-modal");
    const container = document.getElementById("recaptcha-container");
    const openModalButton = document.getElementById("open-recaptcha-modal");
    const closeModalButton = document.getElementById("recaptcha-close");
    const form = document.querySelector("form");
    let recaptchaWidgetId;

  if (!openModalButton) {
        console.error("Tombol 'Daftar Sekarang' tidak ditemukan!");
        return;
    }

    openModalButton.addEventListener("click", () => {
        console.log("Langsung submit tanpa CAPTCHA...");
        form.submit();
    });
});
</script>
<!-- disabled -->
<script> document.addEventListener('contextmenu', function(event) { event.preventDefault(); alert('Right Click Disabled.'); }); </script>
</body>

</html>
<?php /**PATH /home/edunesiaserver/public_html/K2/WEB_KAMPUS/WEB_TEMPLATE/Ijo-Ungu/resources/views/Frontend/Layouts/app2.blade.php ENDPATH**/ ?>