<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class Brosur extends Model
{
    protected $table = 'brosur';
    protected $primaryKey = 'brosur_id';
    protected $fillable = [ 'kategori', 'file'];
    public $timestamps = false; // kalau tidak pakai created_at & updated_at
}