<?php

declare(strict_types=1);

namespace Doctrine\Inflector;

final class Language
{
    public const ENGLISH          = 'english';
    public const FRENCH           = 'french';
    public const NORWEGIAN_BOKMAL = 'norwegian-bokmal';
    public const PORTUGUESE       = 'portuguese';
    public const SPANISH          = 'spanish';
    public const TURKISH          = 'turkish';

    private function __construct()
    {
    }
}
