<?php

namespace Faker\Provider\hr_HR;

class PhoneNumber extends \Faker\Provider\PhoneNumber
{
    protected static $formats = [
        '+385 91 ### ####',
        '+385 92 ### ####',
        '+385 95 ### ####',
        '+385 98 ### ####',
        '+385 99 ### ####',
    ];
}
