<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class TracerStudy extends Model
{
    protected $table = 'tracer_study';
    protected $primaryKey = 'tracer_id';
    
    protected $fillable = [
        'nama',
        'jenis_kelamin',
        'alamat_domisili',
        'jurusan',
        'no_hp',
        'email_pribadi',
        'tanggal_kelulusan',
        'memiliki_sertifikat',
        'sertifikat',
        'nama_instansi',
        'mulai_bekerja',
        'waktu_menunggu',
        'alamat_perusahaan',
        'sosial_media',
        'kesan',
        'saran',
    ];
    
    protected $casts = [
        'sertifikat' => 'array',
    ];
    
    public $timestamps = true;
}