<?php

namespace Faker\Provider\fr_FR;

class Internet extends \Faker\Provider\Internet
{
    protected static $freeEmailDomain = ['gmail.com', 'hotmail.fr', 'yahoo.fr', 'laposte.net', 'free.fr', 'sfr.fr', 'orange.fr', 'club-internet.fr', 'dbmail.com', 'live.com', 'noos.fr', 'tele2.fr', 'wanadoo.fr'];
    protected static $tld = ['com', 'com', 'com', 'net', 'org', 'fr', 'fr', 'fr'];
}
