<div class="md:h-[827px] h-auto flex flex-col items-center md:flex-row justify-center md:gap-x-[100px] py-[100px] md:py-[100px] px-4 md:px-0 bg-cover bg-no-repeat bg-right md:bg-center"
    style="background-image: url('<?php echo e(asset('./images/background/bg-sambutan.png')); ?>');">
    <div class="flex justify-center md:justify-start w-full md:w-auto flex-col items-center">
    <img src="<?php echo e(asset('./images/sambutan/sambutan.png')); ?>" data-aos="fade-up"
        class="h-auto md:w-[1000px] w-[300px] md:h-[500px] max-w-[450px]" style="border-radius:25px;" alt="<?php echo e($settings['nama kampus'] ?? 'Nama kampus tidak tersedia'); ?>">
    <p style="padding-top: 20px; font-size: 14px; color: #fff; text-align: center; font-family: 'Poppins', sans-serif;">
        <!-- <b>Dendi Afrizal, S.AB., M.Ak, </b> -->
    </p>
</div>
    <div class="flex flex-col md:items-start justify-center items-center w-full md:w-[40%] w-[100%] mt-4 md:mt-0"
        data-aos="fade-up">
        <?php if (isset($component)) { $__componentOriginal6a781ab4f935d26ee180d4f6257728a5529effbe = $component; } ?>
<?php $component = $__env->getContainer()->make(App\View\Components\HeaderSection::class, ['subtext' => ' SAMBUTAN REKTOR '.e($settings['nama kampus'] ?? 'Nama kampus tidak tersedia').'','subtextColor' => '#fff','title' => '','titleColor' => '#fff']); ?>
<?php $component->withName('header-section'); ?>
<?php if ($component->shouldRender()): ?>
<?php $__env->startComponent($component->resolveView(), $component->data()); ?>
<?php $component->withAttributes([]); ?>
<?php echo $__env->renderComponent(); ?>
<?php endif; ?>
<?php if (isset($__componentOriginal6a781ab4f935d26ee180d4f6257728a5529effbe)): ?>
<?php $component = $__componentOriginal6a781ab4f935d26ee180d4f6257728a5529effbe; ?>
<?php unset($__componentOriginal6a781ab4f935d26ee180d4f6257728a5529effbe); ?>
<?php endif; ?>
        <p>
           Assalamu’alaikum warahmatullahi wabarakatuh.

            Alhamdulillahi rabbil ‘alamin, segala puji dan syukur ke hadirat Allah SWT. Selamat datang di website resmi Universitas Syekh Yusuf Al Makassari Gowa sebagai sarana keterbukaan informasi, penguatan jejaring akademik, serta pengembangan ilmu pengetahuan yang berlandaskan nilai religius dan kearifan lokal.

            Universitas Syekh Yusuf Al Makassari Gowa berkomitmen mewujudkan visi sebagai perguruan tinggi unggul pada tahun 2028 melalui pendidikan berkualitas, penelitian yang bermanfaat, serta pengabdian kepada masyarakat yang berorientasi pada pemberdayaan dan pembangunan berkelanjutan. Dengan dukungan sumber daya manusia yang kompeten dan semangat kolaborasi, kami bertekad menghasilkan lulusan profesional, berakhlak mulia, berdaya saing, dan adaptif terhadap tantangan global.

            Kami mengundang seluruh masyarakat untuk bersama-sama membangun masa depan melalui pendidikan tinggi yang bermutu dan berintegritas di Universitas Syekh Yusuf Al Makassari Gowa.

            Wassalamu’alaikum warahmatullahi wabarakatuh.<br>
        </p>
        
<a href="/sejarah-singkat" class="bg-yellow-400 hover:bg-primary font-medium py-2 px-3 rounded-md text-sm transition-all  text-center text-white">
    BACA SELENGKAPNYA
</a>
    </div>
</div>
<?php /**PATH /home/edunesiaserver/public_html/K2/WEB_KAMPUS/WEB_TEMPLATE/Ijo-IjoTua/resources/views/Frontend/landingpage/sambutan-section.blade.php ENDPATH**/ ?>