<div class="relative w-full h-screen md:h-screen flex flex-col items-center justify-center overflow-hidden bg-[#019934]" 
     style="height: 107dvh; @media (min-width: 768px) { height: 100vh; }"
     data-aos="fade-up">
  
    <picture class="absolute inset-0 w-full h-full">
        <source 
            srcset="/images/background/hero2.png" 
            media="(max-width: 768px)" 
            class="w-full h-full object-cover"> 
        
        <img 
            src="/images/background/hero1.png" 
            alt="Banner" 
            class="w-full h-full object-cover">
    </picture>

    <div class="relative z-10">
        </div>
</div><?php /**PATH /home/edunesiaserver/public_html/K2/WEB_KAMPUS/WEB_TEMPLATE/Ijo-IjoTua/resources/views/Frontend/landingpage/hero-section.blade.php ENDPATH**/ ?>