<!DOCTYPE html>
<html lang="id">

<head>
    <meta charset="utf-8">

    <meta name="description"
        content="Sekolah Tinggi Ilmu Ekonomi Ganesha (STIE GANESHA) adalah salah satu Perguruan Tinggi Swasta terbaik di Tangerang Selatan. Menyediakan program S1 Manajemen, S1 Akuntansi, dan S2 Manajemen dengan biaya terjangkau mulai dari Rp 400.000/bulan. Cocok untuk karyawan dan profesional yang ingin kuliah sambil bekerja dengan fasilitas modern dan dosen berpengalaman." />
    <meta name="keywords"
        content="Kelas karyawan, Kuliah karyawan, Sekolah Tinggi Ilmu Ekonomi Ganesha, STIE GANESHA, biaya kuliah STIE GANESHA, kampus murah Tangerang Selatan, S1 Manajemen,
         S1 Akuntansi, S2 Manajemen, kuliah online, kuliah fleksibel, kampus akreditasi baik, Edunitas, Universitas Terbuka, STIE Kusuma Negara, STIE Trisakti, STIE Indonesia,
          kampus berkualitas, pendidikan vokasi Tangerang Selatan" />
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<!--ADD GSC-->
    <title><?php echo $__env->yieldContent('title'); ?> &mdash; SEKOLAH TINGGI ILMU EKONOMI GANESHA (Kelas Karyawan)</title>
    <meta property="og:title" content="SEKOLAH TINGGI ILMU EKONOMI GANESHA">
    <meta property="og:type" content="website">
    <meta property="og:url" content="https://stieganesha.info/index.php">
    <meta property="og:image" content="https://stieganesha.info/images/background/hero1.png">
    <meta property="og:image:width" content="1200">
    <meta property="og:image:height" content="630">
    <meta property="og:description"
        content="STIE GANESHA menyediakan pendidikan berkualitas dengan biaya terjangkau. Program S1 dan S2 dengan fasilitas modern dan dosen berpengalaman, ideal untuk mahasiswa yang ingin meningkatkan keahlian di bidang ekonomi dan manajemen." />
    <meta name="twitter:card" content="summary_large_image">
    <meta name="twitter:creator" content="@stieganesha">
    <meta name="twitter:site" content="@stieganesha">
    <meta name="twitter:title" content="SEKOLAH TINGGI ILMU EKONOMI GANESHA">
    <meta name="twitter:description"
        content="Kuliah di STIE GANESHA: Program S1 dan S2 dengan biaya terjangkau, fasilitas lengkap, dan dosen profesional yang siap membimbing mahasiswa menuju kesuksesan." />
    <meta name="twitter:domain" content="https://stieganesha.info/index.php">
    <meta name="twitter:image:src" content="https://stieganesha.info/images/background/hero1.png">
    <meta name="author" content="Tim IT STIE GANESHA">
    <meta http-equiv="cache-control" content="public">
    <meta name="application-name" content="SEKOLAH TINGGI ILMU EKONOMI GANESHA">

    <link rel="apple-touch-icon" sizes="57x57" href="/images/favicons/apple-icon-57x57.png">
    <link rel="apple-touch-icon" sizes="60x60" href="/images/favicons/apple-icon-60x60.png">
    <link rel="apple-touch-icon" sizes="72x72" href="/images/favicons/apple-icon-72x72.png">
    <link rel="apple-touch-icon" sizes="76x76" href="/images/favicons/apple-icon-76x76.png">
    <link rel="apple-touch-icon" sizes="114x114" href="/images/favicons/apple-icon-114x114.png">
    <link rel="apple-touch-icon" sizes="120x120" href="/images/favicons/apple-icon-120x120.png">
    <link rel="apple-touch-icon" sizes="144x144" href="/images/favicons/apple-icon-144x144.png">
    <link rel="apple-touch-icon" sizes="152x152" href="/images/favicons/apple-icon-152x152.png">
    <link rel="apple-touch-icon" sizes="180x180" href="/images/favicons/apple-icon-180x180.png">
    <link rel="icon" type="image/png" sizes="192x192"  href="/images/favicons/android-icon-192x192.png">
    <link rel="icon" type="image/png" sizes="32x32" href="/images/favicons/favicon-32x32.png">
    <link rel="icon" type="image/png" sizes="96x96" href="/images/favicons/favicon-96x96.png">
    <link rel="icon" type="image/png" sizes="16x16" href="/images/favicons/favicon-16x16.png">
    <link rel="manifest" href="/images/favicons/manifest.json">
    <meta name="msapplication-TileColor" content="#ffffff">
    <meta name="msapplication-TileImage" content="/images/favicons/ms-icon-144x144.png">
    <meta name="theme-color" content="#ffffff">
    
    <link href="https://cdn.jsdelivr.net/npm/flowbite@2.5.2/dist/flowbite.min.css" rel="stylesheet" />

    <!-- Add Poppins font -->
    <link rel="preconnect" href="https://fonts.googleapis.com">
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
    <link
        href="https://fonts.googleapis.com/css2?family=Poppins:ital,wght@0,100;0,200;0,300;0,400;0,500;0,600;0,700;0,800;0,900;1,100;1,200;1,300;1,400;1,500;1,600;1,700;1,800;1,900&display=swap"
        rel="stylesheet">

    <script src="https://cdn.tailwindcss.com?plugins=forms,typography,aspect-ratio,line-clamp,container-queries"></script>
</head>

<body class="font-poppins h-screen flex items-center justify-center bg-cover bg-center">
    <?php echo $__env->yieldContent('content'); ?>

    
    <script src="https://cdn.jsdelivr.net/npm/flowbite@2.5.2/dist/flowbite.min.js"></script>
    <!-- Font Awesome for icons (used for Twitter icon) -->
    <script src="https://kit.fontawesome.com/a076d05399.js" crossorigin="anonymous"></script>

    
    <?php echo $__env->yieldPushContent('scripts'); ?>
</body>

</html>
<?php /**PATH /home/edunesiaserver/public_html/K2/WEB_KAMPUS/WEB_TEMPLATE/Ijo-IjoTua/resources/views/auth/app.blade.php ENDPATH**/ ?>